(window.webpackJsonp=window.webpackJsonp||[]).push([[10],{gMlY:function(t,e,n){"use strict";n.r(e),n.d(e,"ConsolePageModule",(function(){return f}));var o=n("ofXK"),r=n("tyNb"),s=n("3Pt+"),c=n("8MG2"),a=n("fXoL"),u=n("ws30"),l=n("+3VT"),i=n("SS/w");function b(t,e){if(1&t&&(a.Pb(0),a.Rb(1,"h2"),a.Cc(2,"Results"),a.Qb(),a.Mb(3,"hr"),a.Mb(4,"app-sql-result",2),a.Ob()),2&t){const t=a.ac();a.zb(4),a.gc("result",t.result)}}let p=(()=>{class t{constructor(t){this.exercise=t}ngOnInit(){}onExecute(t){this.result=t}}return t.\u0275fac=function(e){return new(e||t)(a.Lb(u.a))},t.\u0275cmp=a.Fb({type:t,selectors:[["app-console-page"]],decls:4,vars:2,consts:[["sql","SELECT count(*) AS amount FROM metadata",3,"enableSafeModeTrigger","result"],[4,"ngIf"],[3,"result"]],template:function(t,e){1&t&&(a.Rb(0,"h1"),a.Cc(1,"SQL Console"),a.Qb(),a.Rb(2,"app-sql-input",0),a.Yb("result",(function(t){return e.onExecute(t)})),a.Qb(),a.Ac(3,b,5,1,"ng-container",1)),2&t&&(a.zb(2),a.gc("enableSafeModeTrigger",!0),a.zb(1),a.gc("ngIf",e.result))},directives:[l.a,o.o,i.a],styles:["h1[_ngcontent-%COMP%]{margin-bottom:1.2rem}hr[_ngcontent-%COMP%]{margin-top:.6rem;margin-bottom:.6rem}"]}),t})();var g=n("sWoP");let f=(()=>{class t{}return t.\u0275mod=a.Jb({type:t}),t.\u0275inj=a.Ib({factory:function(e){return new(e||t)},imports:[[o.c,r.g.forChild([{path:"",component:p,pathMatch:"full"}]),s.g,c.a,g.a]]}),t})()}}]);